package cxa.component.mail;

import org.apache.avalon.framework.logger.AbstractLoggable;
import org.apache.avalon.framework.parameters.Parameterizable;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.parameters.ParameterException;
import org.apache.avalon.framework.thread.ThreadSafe;
import java.util.Date;
import java.util.Properties;

import javax.mail.*;
import javax.mail.internet.*;

public class MailComponentImpl
  extends AbstractLoggable
  implements MailComponent, Parameterizable, ThreadSafe{

    protected String mailHost;
    protected String fromAddress;

    /**
     * Parameterizable
     */
    public void parameterize(Parameters parameters)
    throws ParameterException {
        this.mailHost = parameters.getParameter("mailhost");
        this.fromAddress = parameters.getParameter("from");
    }

   public boolean sendMail(String receiver, String subject, String body) {
        if (this.getLogger().isDebugEnabled()) {
            this.getLogger().debug("Trying to send email to " + receiver +
                                   " with subject " + subject + " and content " + body);
        }
        try {
            Properties props = new Properties();
            props.put("mail.smtp.host", this.mailHost);
            Session mailSession = Session.getInstance(props, null);

            MimeMessage pm = new MimeMessage(mailSession);

            // set from
            pm.setFrom(new InternetAddress( this.fromAddress ));
            // set to
            pm.setRecipients(Message.RecipientType.TO,
                                     InternetAddress.parse( receiver ));
             // set subject
            pm.setSubject( subject );
            // set date
            pm.setSentDate(new Date());
            // set content
            pm.setText( body  );
            // send mail
            Transport trans = mailSession.getTransport("smtp");
            Transport.send(pm);
            // success
            return true;
        } catch (Exception any) {
            // failure
            return false;
        }
    }
 }