package cxa.components.source;

import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.logger.AbstractLoggable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.components.source.SourceFactory;
import org.apache.cocoon.components.source.URLSource;
import org.apache.cocoon.environment.Environment;
import org.apache.cocoon.environment.Source;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

public class ZipSourceFactory
extends AbstractLoggable
implements Composable, SourceFactory, ThreadSafe {

    protected ComponentManager manager;

    /**
     * Composable
     */
    public void compose(ComponentManager manager) {
        this.manager = manager;
    }

    public Source getSource(Environment environment, String location)
    throws ProcessingException, MalformedURLException, IOException {
        if ( this.getLogger().isDebugEnabled() ) {
            this.getLogger().debug("Processing " + location);
        }
        int separatorPos = location.indexOf('@');
        if (separatorPos == -1)
            throw new MalformedURLException("@ required in URI: " + location);
        int protocolEnd = location.indexOf("://");
        if (protocolEnd == -1)
            throw new MalformedURLException("URI does not contain '://' : " + location);

        String documentName = location.substring(protocolEnd+3, separatorPos);
        Source archive;
        if (environment == null) {
            archive = new URLSource(new URL(location.substring(separatorPos+1)), this.manager);
        } else {
            try {
                archive = environment.resolve(location.substring(separatorPos+1));
            } catch (SAXException sax) {
                throw new ProcessingException("Unable to resolve " + location.substring(separatorPos+1), sax);
            }
        }
        return new ZipSource(archive, documentName, this.manager);
    }

    public Source getSource(Environment environment, URL base, String location)
    throws ProcessingException, MalformedURLException, IOException {
        return this.getSource(environment, new URL(base, location).toExternalForm());
    }
}
