package cxa.generation;

import org.apache.avalon.excalibur.pool.Poolable;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.ResourceNotFoundException;
import org.apache.cocoon.components.parser.Parser;
import org.apache.cocoon.environment.Source;
import org.apache.cocoon.generation.ComposerGenerator;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public class ZipGenerator
  extends ComposerGenerator
  implements Poolable {

    /**
     * Generates the requested resource.
     */
    public void generate()
    throws SAXException, IOException, ProcessingException {
        int position = this.source.indexOf(".zip");
        if (position == -1) {
            throw new ProcessingException("The URI does not point to a ZIP archive.");
        }
        String zipURI = this.source.substring(0, position + 4);
        String documentName = this.source.substring(position + 5);

        if ( this.getLogger().isDebugEnabled() ) {
            this.getLogger().debug("Reading " + documentName + " from zip archive " + zipURI);
        }

        Source archive = null;
        try {
            archive = this.resolver.resolve( zipURI );

            ZipInputStream zipStream = new ZipInputStream( archive.getInputStream() );
            ZipEntry document = null;
            boolean found = false;
            do {
                document = zipStream.getNextEntry();
                if (document != null) {
                    if ( document.getName().equals(documentName) ) {
                        found = true;
                    } else {
                        // go to next entry
                        zipStream.closeEntry();
                    }
                }
            } while (document != null && found == false);

            if (document == null)
                throw new ResourceNotFoundException("The document " + documentName + " is not in the archive " + zipURI);

            // now we will extract the document and write it into a byte array
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buffer = new byte[8192];
            int length = -1;

            while ( zipStream.available() > 0 ) {
                length = zipStream.read(buffer, 0, 8192);
                if ( length > 0 ) {
                    baos.write(buffer, 0, length);
                }
            }
            zipStream.close();
            baos.flush();

            // now we create the InputSource object for the parser
            ByteArrayInputStream is = new ByteArrayInputStream( baos.toByteArray() );
            Parser parser = null;
            try {
                InputSource source = new InputSource(is);

                parser = (Parser) this.manager.lookup (Parser.ROLE);
                parser.setConsumer( this.xmlConsumer );
                parser.parse( source );
            } catch (ComponentException ce) {
                throw new ProcessingException("Component Parser not found.", ce);
            } finally {
                this.manager.release( parser );
                is.close();
            }
        } finally {
            if (archive != null) archive.recycle();
        }
    }

}
