package cxa.reading;

import org.apache.avalon.excalibur.pool.Poolable;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.ResourceNotFoundException;
import org.apache.cocoon.environment.Source;
import org.apache.cocoon.reading.AbstractReader;
import org.xml.sax.SAXException;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public class ZipReader
  extends AbstractReader
  implements Poolable {

    /**
     * Generates the requested resource.
     */
    public void generate()
    throws SAXException, IOException, ProcessingException {
        int position = this.source.indexOf(".zip");
        if (position == -1) {
            throw new ProcessingException("The URI does not point to a ZIP archive.");
        }
        String zipURI = this.source.substring(0, position + 4);
        String documentName = this.source.substring(position + 5);

        if ( this.getLogger().isDebugEnabled() ) {
            this.getLogger().debug("Reading " + documentName + " from zip archive " + zipURI);
        }

        Source archive = null;
        try {
            archive = this.resolver.resolve( zipURI );

            ZipInputStream zipStream = new ZipInputStream( archive.getInputStream() );
            ZipEntry document = null;
            boolean found = false;
            do {
                document = zipStream.getNextEntry();
                if (document != null) {
                    if ( document.getName().equals(documentName) ) {
                        found = true;
                    } else {
                        // go to next entry
                        zipStream.closeEntry();
                    }
                }
            } while (document != null && found == false);

            if (document == null)
                throw new ResourceNotFoundException("The document " + documentName + " is not in the archive " + zipURI);

            byte[] buffer = new byte[8192];
            int length = -1;

            while ( zipStream.available() > 0 ) {
                length = zipStream.read(buffer, 0, 8192);
                if ( length > 0 ) {
                    this.out.write(buffer, 0, length);
                }
            }
            zipStream.close();
            this.out.flush();

        } finally {
            if (archive != null) archive.recycle();
        }
    }

}
