package cxa.selection;

import org.apache.avalon.framework.logger.AbstractLoggable;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.selection.Selector;
import java.util.Calendar;
import java.util.Map;

/**
 * A <code>Selector</code> that matches a season information
 *
 *  <map:selector name="season" src="cxa.cocoon.selection.SeasonSelector"/>
 *
 *   <map:select type="season">
 *      <map:when test="winter">
 *            <map:transform src="doc2html_wintertime.xsl"/>
 *      </map:when>
 *      <map:otherwise>
 *            <map:transform src="doc2html.xsl"/>
 *       </map:otherwise>
 *   </map:select>
 *
 */
public class SeasonSelector
  extends AbstractLoggable
  implements Selector, ThreadSafe {

    public boolean select(String expression, Map objectModel, Parameters parameters) {
        if ( this.getLogger().isDebugEnabled() ) {
            this.getLogger().debug("Evaluation expression: " + expression);
        }
        boolean result;
        // get the current month
        Calendar currentDate = Calendar.getInstance();
        int month = currentDate.get(Calendar.MONTH);

        // compare
        if ( "summer".equalsIgnoreCase(expression) ) {
            result = (month >= Calendar.JUNE && month <= Calendar.AUGUST);
        } else if ("autumn".equalsIgnoreCase(expression)
                   || "fall".equalsIgnoreCase(expression) ) {
            result = (month >= Calendar.SEPTEMBER && month <= Calendar.NOVEMBER);
        } else if ("winter".equalsIgnoreCase(expression) ) {
            result = (month == Calendar.DECEMBER || month <= Calendar.FEBRUARY);
        } else if ("spring".equalsIgnoreCase(expression) ) {
            result = (month >= Calendar.MARCH && month <= Calendar.MAY);
        } else {
            // unknown expression
            result = false;
        }

        if ( this.getLogger().isDebugEnabled() ) {
            this.getLogger().debug("Result: " + result);
        }

        return result;
    }

}
