package cxa.transformation;

import org.apache.avalon.excalibur.pool.Poolable;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.transformation.AbstractTransformer;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import java.io.IOException;
import java.util.Date;
import java.util.Map;
import java.util.Properties;

import cxa.component.mail.*;

public class SendMailUsingComponentTransformer
extends AbstractTransformer
implements Composable, Poolable {

    public static final String NAMESPACE = "http://cxa/sendmail";
    public static final String SENDMAIL_ELEMENT = "sendmail";
    public static final String MAILTO_ELEMENT = "mailto";
    public static final String MAILSUBJECT_ELEMENT = "mailsubject";
    public static final String MAILBODY_ELEMENT = "mailbody";

    protected static final int MODE_NONE = 0;
    protected static final int MODE_TO = 1;
    protected static final int MODE_SUBJECT = 2;
    protected static final int MODE_BODY = 3;

    protected int mode;
    protected StringBuffer toAddress;
    protected StringBuffer subject;
    protected StringBuffer body;

    protected ComponentManager manager;

    /**
     * Composable
     */
    public void compose(ComponentManager manager) {
        this.manager = manager;
    }

    /**
     * Setup
     */
    public void setup(SourceResolver resolver,
                      Map            objectModel,
                      String         src,
                      Parameters     par)
    throws ProcessingException, SAXException, IOException {
        this.mode = MODE_NONE;
        this.toAddress = new StringBuffer();
        this.subject = new StringBuffer();
        this.body = new StringBuffer();
    }

    public void startElement(String uri, String name, String raw,
                             Attributes attr)
    throws SAXException {
        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("BEGIN startElement uri=" + uri +
               ", name=" + name + ", raw=" + raw + ", attr=" + attr);
        }

        if (uri != null && uri.equals(NAMESPACE) ) {
            if (name.equals(SENDMAIL_ELEMENT) == true) {
                // No need to do anything here
            } else if (name.equals(MAILTO_ELEMENT) == true) {
                this.mode = MODE_TO;
            } else if (name.equals(MAILSUBJECT_ELEMENT) == true) {
                this.mode = MODE_SUBJECT;
            } else if (name.equals(MAILBODY_ELEMENT) == true) {
                this.mode = MODE_BODY;
            } else {
                throw new SAXException("Unknown element " + name);
            }
        } else {
            // Not for us
            super.startElement(uri, name, raw, attr);
        }

        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("END startElement");
        }
    }

    public void endElement(String uri, String name, String raw)
    throws SAXException {
        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("BEGIN endTransformingElement uri=" + uri +
                              ", name=" + name + ", raw=" + raw);
        }

        if (uri != null && uri.equals(NAMESPACE) ) {
            if (name.equals(SENDMAIL_ELEMENT) == true) {
                if (this.getLogger().isInfoEnabled() == true) {
                    this.getLogger().info("Mail contents: To: "+ this.toAddress +
                                     ", Subject: " + this.subject +
                                     ", Body: "+ this.body);
                }

                String text;
                MailComponent mail = null;
                try {
                    mail = (MailComponent) this.manager.lookup(MailComponent.ROLE);
                    if (mail.sendMail(this.toAddress.toString(),
                             this.subject.toString(),
                             this.body.toString()) ) {
                        text = "Sending mail to " + this.toAddress + " was successful.";
                    } else {
                        text = "Sending mail to " + this.toAddress + " failed!";
                    }
                    // create SAX events for success/failure
                    super.startElement(NAMESPACE, "sendmail", "sendmail", new AttributesImpl());
                    super.characters(text.toCharArray(), 0, text.length());
                    super.endElement(NAMESPACE, "sendmail", "sendmail");
                } catch (ComponentException ce) {
                    throw new SAXException("Unable to lookup mail component.", ce);
                } finally {
                    this.manager.release(mail);
                }
            } else if (name.equals(MAILTO_ELEMENT) == true) {
                // mailto received
                this.mode = MODE_NONE;
            } else if (name.equals(MAILSUBJECT_ELEMENT) == true) {
                this.mode = MODE_NONE;
            } else if (name.equals(MAILBODY_ELEMENT) == true) {
                this.mode = MODE_NONE;
            } else {
                throw new SAXException("Unknown element " + name);
            }
        } else {
            // Not for us
            super.endElement(uri, name, raw);
        }

        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("END endElement");
        }
    }

    public void characters(char[] buffer, int start, int length)
    throws SAXException {
        switch (this.mode) {
            case MODE_NONE : super.characters(buffer, start, length);
                             break;
            case MODE_TO : this.toAddress.append(buffer, start, length);
                           break;
            case MODE_SUBJECT : this.subject.append(buffer, start, length);
                           break;
            case MODE_BODY : this.body.append(buffer, start, length);
                           break;
        }
    }
}